@@ -1,7 +1,7 @@
 """
 """
 
-from typing import NoReturn, Union
+from typing import Union
 
 import numpy as np
 from scipy.stats import norm, beta, binom, ncx2
@@ -165,6 +165,8 @@ def _compute_confidence_interval(
             str(sides),
         )
     elif confint_type.lower() in ["wilson-cc", "newcombe-cc"]:
+        # https://corplingstats.wordpress.com/2019/04/27/correcting-for-continuity/
+        # equation (6) and (6')
         e = 2 * n_total * ratio + z**2
         f = z**2 - 1 / n_total + 4 * n_total * ratio * neg_ratio
         g = 4 * ratio - 2
@@ -496,13 +498,13 @@ _method_aliases = _type_aliases
 
 
 @deprecated(version="0.0.4", reason="Use `list_confidence_interval_methods` instead.")
-def list_confidence_interval_types() -> NoReturn:
+def list_confidence_interval_types() -> None:
     """ """
 
     print("\n".join(_supported_types))
 
 
-def list_confidence_interval_methods() -> NoReturn:
+def list_confidence_interval_methods() -> None:
     """ """
 
     print("\n".join(_supported_types))
